# Name

pacboy extension - Pacboy extension modules


# Description

Pacboy can be extended with build support for binary Pacman repos and third-party sources. This requires the implementation of a few classes in a Python 3 module. Basically, Pacboy needs access to essential package metadata (name, version, dependencies and a few others) along with a way to get it and the PKGBUILDs.


# Security

Under no circumstances should you run makepkg or mksrcinfo on third-party PKGBUILDS without prompting the user. Figure out another way to get the metadata that you need, such as requesting the PKGBUILD provide to provide SRCINFO files (which should be easy to do).

The only PKGBUILDs that may be run with makepkg or mksrcinfo are the official PKGBUILDs from Arch Linux as these are implicity trusted by pacman and ABS.

If your extension parses your own PKGBUILDs without third-party input then that may be acceptable, but this must be made clear to the user when distributing your extension.


# BuildExtender

To add build support to a Pacman binary repo (official or custom), i.e. one that you can already use with vanilla Pacman, the following must be subclassed:

* XCPF.Pkg.Pkg
* XCPF.RepoWrapper.BuildExtender
* XCPF.RepoWrapper.BuildExtenderFactory

Check the documentation in the source code for each class's methods. For a reference implementation check XCPF.PacboyExtXac which adds build support for the pacman binary repos on [http://xyne.archlinux.ca/repos/](xyne.archlinux.org). This implementation is based on downloadable SRCINFO and PKGBUILD archives and is relatively simple.

If the target repo website is able to provide downloadable SRCINFO files and PKGBUILD archives then you can just subclass `XacBuildExtender` and override the `srcinfo_url` and `pkgbuild_archive_url` methods.

XCPF.PacmanRepoWrapper may also be instructive but it is far more complicated due to the way official package metadata is spread across multiple interfaces (e.g. the website interface must be accessed to determine the PKGBASE, the website must be scraped for PKGBUILD downloads, etc.) In the future this will hopefully be simplified.

# RepoWrapper

To create a completely new repo interface to a third-party repo that requires package building (e.g. the AUR), the following must be subclassed:

* XCPF.Pkg.Pkg
* XCPF.RepoWrapper.RepoWrapper
* XCPF.RepoWrapper.RepoWrapperFactory

The source code comments are the best source of information. For a reference implementation, check AUR.RepoWrapper in `python3-aur`. Again, due to shortcomings in the AUR's interface this implementation is more complicated than it should be but it will hopefully be simplified with a future AUR update.

# Tips

.SRCINFO is the current common format for package metadata. If possible, retrieve or generate .SRCINFO files and use the parser in XCPF to get the package metadata. All that's needed then is to generate the PKGBUILD and related files. The Xac extension provides SRCINFO files and PKGBUILD archives via the web interface, for example.

Keep in mind that there is a distinction between package bases and package names. If a repo provides split packages then SRCINFO and PKGBUILDs should be provided for each package base and the subclass of XCPF.Pkg.Pkg must be able to determine the package base.

The download job data structures are a little tricky at first but very versatile. They provide a way to group multiple downloads together as a single job that either succeeds or fails, with post-download transformation of the output if necessary (e.g. archive extraction, post-download PKGUILD generation). Check the Xac extension for the most accessible example along with the help functions such as `source_array_download_jobs` in XCPF.RepoWrapper.

Cache downloads when possible to speed up subsequent operations. For example, if an archive needs to be downloaded with the PKGBUILD to generate SRCINFO, cache the archive in case the PKGBUILD is requested in a later operation. The environment object provides the time-to-live parameter.
